#ifndef INTERVALLE_H
#define INTERVALLE_H

#include <limits>

class BornesInvalides {};
class ValeurIncoherente {};

//
// Contrat de T:
// - constructeur par dfaut
// - constructeur par copie
// - existence de numeric_limits<T>::min()
// - existence de numeric_limits<T>::max()
// - support usuel des oprateurs ++ et --
//
template <class T>
   struct Intervalle
   {
      using value_type = T;
      const value_type minval;
      const value_type maxval;
      bool est_inclus(const value_type &val) const
         { return minval <= val && val <= maxval; }
      Intervalle()
         : minval{ std::numeric_limits<value_type>::min() },
           maxval{ std::numeric_limits<value_type>::max() }
      {
      }
      Intervalle(const value_type &minval)
         : minval{ minval }, maxval{ std::numeric_limits<value_type>::max() }
      {
      }
      Intervalle(const value_type &minval, const value_type &maxval)
         : minval{ minval }, maxval{ maxval } 
      {
         if (minval > maxval) throw BornesInvalides{};
      }
   };

#endif
