#ifndef SINGLETON_CONTEXT_H
#define SINGLETON_CONTEXT_H

#include "SingletonContextFwd.h"
#include "Incopiable.h"
#include "type_list.h"
#include <type_traits>

template <class T>
   class ConteneurSingleton
   {
   public:
      T& get()
      {
         static T singleton;
         return singleton;
      }
   };

template <class T, class ... Q>
   class ConteneurSingleton<type_list<T, Q...>>
      : public ConteneurSingleton<type_list<Q...>>
   {
   public:
      using singleton_type = T;
   private:
      SingletonWrapper<singleton_type> singleton_;
      template <class U>
         U& ParentGet()
            { return ConteneurSingleton<type_list<Q...>>::get<U>(); }
      template <class T>
         struct searcher {};
      class Found {};
      class NotFound {};
      template <class U>
         U& search_for(searcher<U>, Found)
            { return singleton_.get(); };
      template <class U>
         U& search_for(searcher<U>, NotFound)
            { return ParentGet<U>(); };
   public:
      template <class U>
         U& get()
         {
            return search_for(searcher<U>{}, std::conditional_t<
               std::is_same<T, U>::value, Found, NotFound
            >{});
         }
   };

template <class TList>
   class SingletonContext
      : ConteneurSingleton<TList>, Incopiable
   {
      static SingletonContext ctx;
      SingletonContext() = default;
   public:
      static SingletonContext& get()
         { return ctx; }
      template <class T>
         T & get_singleton()
            { return ConteneurSingleton<TList>::get<T>(); }
   };
template <class TList>
   SingletonContext<TList> SingletonContext<TList>::ctx;

#endif
